<?php

namespace FiloBlu\ActiveCampaign\Helper;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Module\Manager;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\StoreManagerInterface;


/**
 * Class Data
 * @package FiloBlu\ActiveCampaign\Helper
 */
class Data extends AbstractHelper {

    //GENERAL
    const FB_ACTIVECAMPAIGN_GENERAL_ENABLE = 'fb_activecampaign_general/general/enable';
    const FB_ACTIVECAMPAIGN_GENERAL_REMOVE_PUB = 'fb_activecampaign_general/general/remove_pub';
    //OBSERVERS
    const FB_ACTIVECAMPAIGN_OBSERVER_ENABLE = 'fb_activecampaign_general/observers/enable';
    const FB_ACTIVECAMPAIGN_OBSERVER_SUBSCRIPTION_CHANGE = 'fb_activecampaign_general/observers/enable_subscription_change';
    const FB_ACTIVECAMPAIGN_OBSERVER_CUSTOMER_CHANGE = 'fb_activecampaign_general/observers/enable_customer_change';
    //REST
    const FB_ACTIVECAMPAIGN_REST_ENABLE = 'fb_activecampaign_general/rest/enable';
    const FB_ACTIVECAMPAIGN_REST_BASE_URL = 'fb_activecampaign_general/rest/base_url';
    const FB_ACTIVECAMPAIGN_REST_TOKEN = 'fb_activecampaign_general/rest/token';
    const FB_ACTIVECAMPAIGN_REST_WEBHOOK_CODE = 'fb_activecampaign_general/rest/webhook_code';
    const FB_ACTIVECAMPAIGN_REST_UNSUBSCRIBE_WEBHOOK_ENABLED = 'fb_activecampaign_general/rest/enable_unsubscriber_webhook';
    //GENDER
    const FB_ACTIVECAMPAIGN_CUSTOMER_MAPPING_GENDER = 'fb_activecampaign_customer/mapping/gender';
    //SUBSCRIBER
    const FB_ACTIVECAMPAIGN_CUSTOMER_MAPPING_SUBSCRIBER = 'fb_activecampaign_customer/mapping/subscriber';
    //CUSTOMER
    const FB_ACTIVECAMPAIGN_CUSTOMER_ENABLE = 'fb_activecampaign_customer/customer/enable';
    const FB_ACTIVECAMPAIGN_CUSTOMER_MAPPING_CUSTOMER = 'fb_activecampaign_customer/mapping/customer';
    const FB_ACTIVECAMPAIGN_CUSTOMER_MAPPING_BILLING = 'fb_activecampaign_customer/mapping/customer_billing_address';
    const FB_ACTIVECAMPAIGN_CUSTOMER_MAPPING_SHIPPING = 'fb_activecampaign_customer/mapping/customer_shipping_address';
    //QUOTE
    const FB_ACTIVECAMPAIGN_SALES_QUOTE_ENABLE = 'fb_activecampaign_sales/quote/enable';
    const FB_ACTIVECAMPAIGN_SALES_QUOTE_EXPORT_ONLY_SIMPLE = 'fb_activecampaign_sales/quote/export_only_simple';
    const FB_ACTIVECAMPAIGN_SALES_QUOTE_ABANDONED_CART_TIME = 'fb_activecampaign_sales/quote/abandoned_cart_time';
    //ORDER
    const FB_ACTIVECAMPAIGN_SALES_ORDER_ENABLE = 'fb_activecampaign_sales/order/enable';
    const FB_ACTIVECAMPAIGN_SALES_ORDER_EXPORT_ONLY_SIMPLE = 'fb_activecampaign_sales/order/export_only_simple';
    const FB_ACTIVECAMPAIGN_SALES_ORDER_STATUSES = 'fb_activecampaign_sales/order/order_statuses_to_export';
    //LIST
    const FB_ACTIVECAMPAIGN_LIST_CUSTOMER = 'fb_activecampaign_general/lists/customer';
    const FB_ACTIVECAMPAIGN_LIST_CUSTOMER_ENABLED = 'fb_activecampaign_general/lists/customer_list_enabled';
    const FB_ACTIVECAMPAIGN_LIST_SUBSCRIBER = 'fb_activecampaign_general/lists/subscriber';
    //CONNECTION
    const FB_ACTIVECAMPAIGN_CONNECTION_ID = 'fb_activecampaign_general/connection/id';
    const FB_ACTIVECAMPAIGN_CONNECTION_NAME = 'fb_activecampaign_general/connection/name';
    const FB_ACTIVECAMPAIGN_CONNECTION_EXTERNAL_ID = 'fb_activecampaign_general/connection/external_id';
    const FB_ACTIVECAMPAIGN_CONNECTION_URL = 'fb_activecampaign_general/connection/url';
    const FB_ACTIVECAMPAIGN_CONNECTION_LOGO = 'fb_activecampaign_general/connection/logo';
    //MODULE
    const FB_ACTIVECAMPAIGN_NEWSLETTER_MODULE = 'FiloBlu_Newsletter';
    const FB_ACTIVECAMPAIGN_REFILO_MODULE = 'FiloBlu_Refilo';

    /**
     * @var SerializerInterface
     */
    public $serializer;
    /**
     * @var WriterInterface
     */
    protected $configWriter;
    /**
     * @var Manager
     */
    protected $moduleManager;
    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;
    /**
     * @var ObjectManagerInterface
     */
    protected $objectManager;
    /**
     * @var null|mixed
     */
    protected $refiloHelperData = null;
    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;


    /**
     * Data constructor.
     * @param Context $context
     * @param SerializerInterface $serializer
     * @param WriterInterface $configWriter
     * @param Manager $moduleManager
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        Context $context,
        SerializerInterface $serializer,
        WriterInterface $configWriter,
        Manager $moduleManager,
        StoreManagerInterface $storeManager,
        ObjectManagerInterface $objectManager,
        ProductRepositoryInterface $productRepository
    )
    {
        parent::__construct($context);
        $this->serializer = $serializer;
        $this->configWriter = $configWriter;
        $this->moduleManager = $moduleManager;
        $this->storeManager = $storeManager;
        $this->objectManager = $objectManager;
        $this->productRepository = $productRepository;
    }

    /**
     * @return bool
     */
    public function isEnabled(): bool
    {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_GENERAL_ENABLE);
    }

    /**
     * @return bool
     */
    public function isObserverEnabled(): bool {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_OBSERVER_ENABLE);
    }

    /**
     * @return bool
     */
    public function isCustomerSaveObserverEnabled(): bool
    {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_OBSERVER_CUSTOMER_CHANGE);
    }

    /**
     * @return bool
     */
    public function isSubscriptionChangeObserverEnabled(): bool {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_OBSERVER_SUBSCRIPTION_CHANGE);
    }

    /**
     * @return bool
     */
    public function isRestEnabled(): bool {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_REST_ENABLE);
    }

    /**
     * @return mixed
     */
    public function getRestBaseUrl() {
        return $this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_REST_BASE_URL);
    }

    /**
     * @return mixed
     */
    public function getRestToken() {
        return $this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_REST_TOKEN);
    }

    /**
     * @return bool
     */
    public function isQuoteEnabled(): bool
    {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_SALES_QUOTE_ENABLE);
    }

    /**
     * @return bool
     */
    public function quoteExportOnlySimple(): bool
    {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_SALES_QUOTE_EXPORT_ONLY_SIMPLE);
    }

    /**
     * @return mixed
     */
    public function getAbandonedCartTime() {
        return $this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_SALES_QUOTE_ABANDONED_CART_TIME);
    }

    /**
     * @return bool
     */
    public function isOrderEnabled(): bool
    {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_SALES_ORDER_ENABLE);
    }

    /**
     * @return bool
     */
    public function isCustomerListEnabled(): bool
    {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_LIST_CUSTOMER_ENABLED);
    }

    /**
     * @return bool
     */
    public function orderExportOnlySimple(): bool
    {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_SALES_ORDER_EXPORT_ONLY_SIMPLE);
    }

    /**
     * @return mixed
     */
    public function getOrderStatuses() {
        return $this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_SALES_ORDER_STATUSES);
    }

    /**
     * @return bool
     */
    public function isCustomerEnabled(): bool
    {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_CUSTOMER_ENABLE);
    }

    /**
     * @return mixed
     */
    public function getGenderMapping(): array
    {
        return $this->getSerializedValue($this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_CUSTOMER_MAPPING_GENDER),'gender');
    }

    /**
     * @return mixed
     */
    public function getCustomerMapping(): array
    {
        return $this->getSerializedValue($this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_CUSTOMER_MAPPING_CUSTOMER), 'customer');
    }

    /**
     * @return mixed
     */
    public function getCustomerBillingAddressMapping(): array
    {
        return $this->getSerializedValue($this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_CUSTOMER_MAPPING_BILLING), 'customer_billing_address');
    }

    /**
     * @return mixed
     */
    public function getCustomerShippingAddressMapping(): array
    {
        return $this->getSerializedValue($this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_CUSTOMER_MAPPING_SHIPPING), 'customer_shipping_address');
    }

    /**
     * @return mixed
     */
    public function getSubscriberMapping(): array
    {
        return $this->getSerializedValue($this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_CUSTOMER_MAPPING_SUBSCRIBER), 'subscriber');
    }

    /**
     * @param $value
     * @return array
     */
    protected function getSerializedValue($value,$entity): array
    {
        $map = [];
        if(empty($value)){
            return $map;
        }
        $datas = $this->serializer->unserialize($value);

        foreach ($datas as $data) {
            $map[$data[$entity.'_magento_field']] = $data[$entity.'_activecampaign_field'];
        }
        return $map;
    }

    /**
     * @param $value
     * @return bool
     */
    public function isAdditional($value): bool
    {
        $val = substr($value,0,1);
        return $val === '_';
    }

    /**
     * @param $value
     * @return string
     */
    public function snakeToCamelCase($value): string
    {
        return ucfirst(str_replace(' ', '', ucwords(str_replace('_', ' ', strtolower($value)))));
    }

    /**
     * @param array $conditions
     * @return string
     */
    public function arrayToWhereString(array $conditions): string
    {
        $whereArray = [];
        $where = '';
        if(count($conditions)==0){
            return $where;
        }
        foreach ($conditions as $key => $val) {
            $whereArray[] = $key." = '".$val."'";
        }
        $where = implode(' AND ',$whereArray);
        return $where;
    }

    /**
     * @param $id
     */
    public function writeConnectionId($id) {
        $this->configWriter->save(self::FB_ACTIVECAMPAIGN_CONNECTION_ID,$id);
    }

    /**
     * @return mixed
     */
    public function getConnectionId() {
        return $this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_CONNECTION_ID);
    }

    /**
     * @return mixed
     */
    public function getConnectionName() {
        return $this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_CONNECTION_NAME);
    }

    /**
     * @return mixed
     */
    public function getConnectionExternalId() {
        return $this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_CONNECTION_EXTERNAL_ID);
    }

    /**
     * @return mixed
     */
    public function getConnectionUrl() {
        return $this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_CONNECTION_URL);
    }

    /**
     * @return mixed
     */
    public function getConnectionLogo() {
        return $this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_CONNECTION_LOGO);
    }

    /**
     * @return mixed
     */
    public function getCustomerList() {
        return $this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_LIST_CUSTOMER);
    }

    /**
     * @return mixed
     */
    public function getSubscriberList() {
        return $this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_LIST_SUBSCRIBER);
    }

    /**
     * @return bool
     */
    public function isNewsletterExtensionEnabled(): bool
    {
        return $this->moduleManager->isEnabled(self::FB_ACTIVECAMPAIGN_NEWSLETTER_MODULE);
    }

    /**
     * @param $price
     * @return float|int
     */
    public function formatPrice($price){
        return (float)$price * 100;
    }

    /**
     * @return bool
     */
    public function removePub(): bool
    {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_GENERAL_REMOVE_PUB);
    }

    /**
     * @param $storeId
     * @return string
     * @throws NoSuchEntityException
     */
    public function getMediaUrl($storeId) {
        $store = $this->storeManager->getStore($storeId);
        $storeUrl = $store->getBaseUrl(UrlInterface::URL_TYPE_MEDIA);
        if($this->removePub()) {
            $storeUrl = str_replace('/pub/','/',$storeUrl);
        }
        return $storeUrl;
    }

    /**
     * @return bool
     */
    public function isUnsubscriberWebhookEnabled() {
        return $this->scopeConfig->isSetFlag(self::FB_ACTIVECAMPAIGN_REST_UNSUBSCRIBE_WEBHOOK_ENABLED);
    }

    /**
     * @param $code
     * @return bool
     */
    public function checkWebhookCode($code) {
        $webhook_code = $this->scopeConfig->getValue(self::FB_ACTIVECAMPAIGN_REST_WEBHOOK_CODE);
        if($webhook_code!=$code){
            return false;
        }
        return true;
    }

    /**
     * @param $date
     * @return false|string
     */
    public function getACDatetimeFormat($date) {
        return date('c',strtotime($date));
    }

    /**
     * @return bool
     */
    public function isRefiloExtensionEnabled() {
        return $this->moduleManager->isEnabled(self::FB_ACTIVECAMPAIGN_REFILO_MODULE);
    }

    /**
     * @param $product
     */
    public function getProductUrl($product,$store_id) {
        if($this->isRefiloExtensionEnabled()) {
            $helper = $this->getRefiloHelperData();
            if(method_exists($helper,'getProductStorefrontUrl')){
                return $helper->getProductStorefrontUrl($product,$store_id);
            }
        }
        return $product->getProductUrl();
    }

    /**
     * @return mixed|null
     */
    protected function getRefiloHelperData() {
        if(empty($this->refiloHelperData)){
            $this->refiloHelperData = $this->objectManager->create('FiloBlu\Refilo\Helper\Data');
        }
        return $this->refiloHelperData;
    }

    /**
     * @param $productId
     */
    public function getProduct($productId) {
        return $this->productRepository->getById($productId);
    }

}
